/*
 * FLUENDO S.A.
 * Copyright (C) <2005 - 2011>  <support@fluendo.com>
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gst/gst.h>
#include "mp3tl-priv.h"
#include "table-dewindow.h"
#include "mp3-cos-tables.h"

#ifdef USE_LIBOIL
#include <liboil/liboil.h>
#endif

#define OPT_SYNTH

void
III_subband_synthesis (mp3tl * tl, frame_params * fr_ps,
    gfloat hybridOut[SBLIMIT][SSLIMIT], gint channel,
    short samples[SSLIMIT][SBLIMIT])
{
  gint ss, sb;
  gfloat polyPhaseIn[SBLIMIT];  /* PolyPhase Input. */

  for (ss = 0; ss < 18; ss++) {
    /* Each of the 32 subbands has 18 samples. On each iteration, we take
     * one sample from each subband, (32 samples), and use a 32 point DCT
     * to perform matrixing, and copy the result into the synthesis
     * buffer fifo. */
    for (sb = 0; sb < SBLIMIT; sb++) {
      polyPhaseIn[sb] = hybridOut[sb][ss];
    }

    mp3_SubBandSynthesis (tl, fr_ps, polyPhaseIn, channel,
        &(tl->pcm_sample[channel][ss][0]));
  }
}

#ifndef USE_IPP

#ifdef OPT_SYNTH

#define INV_SQRT_2 (7.071067811865474617150084668537e-01)

static inline void
MPG_DCT_8 (gfloat in[8], gfloat out[8])
{
  gfloat even_in[4];
  gfloat odd_in[4], odd_out[4];
  gfloat tmp[6];

  /* Even indices */
  even_in[0] = in[0] + in[7];
  even_in[1] = in[3] + in[4];
  even_in[2] = in[1] + in[6];
  even_in[3] = in[2] + in[5];

  tmp[0] = even_in[0] + even_in[1];
  tmp[1] = even_in[2] + even_in[3];
  tmp[2] = (even_in[0] - even_in[1]) * synth_cos64_table[7];
  tmp[3] = (even_in[2] - even_in[3]) * synth_cos64_table[23];
  tmp[4] = (gfloat) ((tmp[2] - tmp[3]) * INV_SQRT_2);

  out[0] = tmp[0] + tmp[1];
  out[2] = tmp[2] + tmp[3] + tmp[4];
  out[4] = (gfloat) ((tmp[0] - tmp[1]) * INV_SQRT_2);
  out[6] = tmp[4];

  /* Odd indices */
  odd_in[0] = (in[0] - in[7]) * synth_cos64_table[3];
  odd_in[1] = (in[1] - in[6]) * synth_cos64_table[11];
  odd_in[2] = (in[2] - in[5]) * synth_cos64_table[19];
  odd_in[3] = (in[3] - in[4]) * synth_cos64_table[27];

  tmp[0] = odd_in[0] + odd_in[3];
  tmp[1] = odd_in[1] + odd_in[2];
  tmp[2] = (odd_in[0] - odd_in[3]) * synth_cos64_table[7];
  tmp[3] = (odd_in[1] - odd_in[2]) * synth_cos64_table[23];
  tmp[4] = tmp[2] + tmp[3];
  tmp[5] = (gfloat) ((tmp[2] - tmp[3]) * INV_SQRT_2);

  odd_out[0] = tmp[0] + tmp[1];
  odd_out[1] = tmp[4] + tmp[5];
  odd_out[2] = (gfloat) ((tmp[0] - tmp[1]) * INV_SQRT_2);
  odd_out[3] = tmp[5];

  out[1] = odd_out[0] + odd_out[1];
  out[3] = odd_out[1] + odd_out[2];
  out[5] = odd_out[2] + odd_out[3];
  out[7] = odd_out[3];
}

static inline void
MPG_DCT_16 (gfloat in[16], gfloat out[16])
{
  int i;
  gfloat even_in[8], even_out[8], odd_in[8], odd_out[8];

  for (i = 0; i < 8; i++) {
    even_in[i] = in[i] + in[15 - i];
    odd_in[i] = (in[i] - in[15 - i]) * synth_cos64_table[4 * i + 1];
#if 0
    g_print ("i = %d, index %d calc %.06f table %.06f\n", i, 4 * i + 1,
        (1.0 / (2.0 * cos ((2 * i + 1) * (M_PI / (32))))),
        synth_cos64_table[4 * i + 1]);
#endif
  }

  MPG_DCT_8 (even_in, even_out);
  MPG_DCT_8 (odd_in, odd_out);

  for (i = 0; i < 8; i++) {
    out[2 * i] = even_out[i];
  }

  for (i = 0; i < 7; i++) {
    out[2 * i + 1] = odd_out[i] + odd_out[i + 1];
  }
  out[15] = odd_out[7];
}

static inline void
MPG_DCT_32 (gfloat in[32], gfloat out[32])
{
  int i;
  gfloat even_in[16], even_out[16], odd_in[16], odd_out[16];

  for (i = 0; i < 16; i++) {
    even_in[i] = in[i] + in[31 - i];
    odd_in[i] = (in[i] - in[31 - i]) * synth_cos64_table[2 * i];
  }

  MPG_DCT_16 (even_in, even_out);
  MPG_DCT_16 (odd_in, odd_out);

  for (i = 0; i < 15; i++) {
    out[2 * i] = even_out[i];
    out[2 * i + 1] = odd_out[i] + odd_out[i + 1];
  }
  out[30] = even_out[15];
  out[31] = odd_out[15];
}


#if defined(USE_ARM_VFP) && defined(__VFP_FP__) && !defined(__SOFTFP__)
#define CONVERT_TO_INTEGER(v,d)                                               \
    "  fcmpezs     " v "                 \n\t"                                \
    "  fmstat                            \n\t"                                \
    "  faddsgt     " d ", " v ", s2      \n\t"  /* v > 0 ? d = v + 0.5 */     \
    "  fsubsle     " d ", " v ", s2      \n\t"  /* v <= 0 ? d = v - 0.5 */    \
    "  ftosizs     " d ", " d "          \n\t"

#define WRITE_FOUR_SAMPLES                                                    \
    "  fmrrs       r4, r5, {s4, s5}      \n\t"                                \
    "  fmrrs       r6, r7, {s6, s7}      \n\t"                                \
    "  ssat        r4, #16, r4           \n\t"                                \
    "  ssat        r5, #16, r5           \n\t"                                \
    "  ssat        r6, #16, r6           \n\t"                                \
    "  ssat        r7, #16, r7           \n\t"                                \
    "  strh        r4, [%[samp], #0]     \n\t"                                \
    "  strh        r5, [%[samp], #2]     \n\t"                                \
    "  strh        r6, [%[samp], #4]     \n\t"                                \
    "  strh        r7, [%[samp], #6]     \n\t"                                \
    "  add         %[samp], %[samp], #8  \n\t"

static inline void
mp3_dewindow_output (gfloat *uvec, short *samples, gfloat* window)
{
  gfloat k[] = { SCALE, 0.5 };

  __asm__ volatile (
      "fldmias       %[k], {s1-s2}        \n\t"
      "fmrx          ip, fpscr            \n\t" /* read fpscr register into arm */
      "mov           r8, #7               \n\t"
      "orr           r8, ip, r8, lsl #16  \n\t" /* set vector lenght to 8 */
      "fmxr          fpscr, r8            \n\t"
      "mov           r8, #4               \n\t" /* i = 4 */
      "0:                                 \n\t"
      "  mov         r6, %[win]           \n\t" /* r6 = &win[0] */
      "  mov         r4, %[uvec]          \n\t" /* r4 = &uvec[0] */
      "  fldmias     %[win]!, {s8-s15}    \n\t" /* win[0..7] */
      "  fldmias     %[uvec]!, {s16-s23}  \n\t" /* uvec[0..7] */
      "  fmuls       s24, s8, s16         \n\t" /* s24..s31 = win[0..7] * uvec[0..7] */
      /* Multiply and accumulate loop */
      "  mov         r9, #15              \n\t" /* j = 15 */
      "1:                                 \n\t"
      "  add         r6, r6, #128         \n\t" /* r6 = &win[32] */
      "  add         r4, r4, #128         \n\t" /* r4 = &uvec[32] */
      "  fldmias     r6, {s8-s15}         \n\t" /* win[32..39] */
      "  fldmias     r4, {s16-s23}        \n\t" /* uvec[32..39] */
      "  fmacs       s24, s8, s16         \n\t" /* s24..s31 += win[32..39] * uvec[32..39] */
      "  subs        r9, r9, #1           \n\t" /* j-- */
      "  bne         1b                   \n\t"
      /* Scale results */
      "  fmuls       s8, s24, s1          \n\t" /* uvec[0..7] *= SCALE */
      /* Convert samples into integers and write them */
      CONVERT_TO_INTEGER("s8","s4")
      CONVERT_TO_INTEGER("s9","s5")
      CONVERT_TO_INTEGER("s10","s6")
      CONVERT_TO_INTEGER("s11","s7")
      WRITE_FOUR_SAMPLES
      CONVERT_TO_INTEGER("s12","s4")
      CONVERT_TO_INTEGER("s13","s5")
      CONVERT_TO_INTEGER("s14","s6")
      CONVERT_TO_INTEGER("s15","s7")
      WRITE_FOUR_SAMPLES
      "  subs        r8, r8, #1           \n\t" /* i-- */
      "  bne         0b                   \n\t"
      "fmxr          fpscr, ip            \n\t" /* restore original fpscr */
    : [win] "+&r" (window),
      [samp] "+&r" (samples),
      [uvec] "+&r" (uvec)
    : [k] "r" (k)
    : "memory", "cc", "r4", "r5", "r6", "r7", "r8", "r9", "ip",
      "s1", "s2", "s4", "s5", "s6", "s7",
      "s8", "s9", "s10", "s11", "s12", "s13", "s14", "s15",
      "s16", "s17", "s18", "s19", "s20", "s21", "s22", "s23",
      "s24", "s25", "s26", "s27", "s28", "s29", "s30", "s31"
  );
}
#else
static inline void
mp3_dewindow_output (gfloat *u_vec, short *samples, gfloat* window)
{
  gint i;
  gfloat *u_vec0;

  /* dewindowing */
#ifdef USE_LIBOIL
  oil_multiply_f32 (u_vec, u_vec, window, HAN_SIZE);
#else
  for (i = 0; i < HAN_SIZE; i++)
    u_vec[i] *= dewindow[i];
#endif

  /* Now calculate 32 samples */
#ifdef USE_LIBOIL
  u_vec0 = u_vec + 32;
  for (i = 0; i < 15; i++) {
    oil_add_f32 (u_vec, u_vec, u_vec0, 32);
    u_vec0 += 32;
  }
#else
  for (i = 0; i < 32; i++) {
    gfloat sum;
    u_vec0 = u_vec + i;
    sum = u_vec0[1 << 5];
    sum += u_vec0[2 << 5];
    sum += u_vec0[3 << 5];
    sum += u_vec0[4 << 5];
    sum += u_vec0[5 << 5];
    sum += u_vec0[6 << 5];
    sum += u_vec0[7 << 5];
    sum += u_vec0[8 << 5];
    sum += u_vec0[9 << 5];
    sum += u_vec0[10 << 5];
    sum += u_vec0[11 << 5];
    sum += u_vec0[12 << 5];
    sum += u_vec0[13 << 5];
    sum += u_vec0[14 << 5];
    sum += u_vec0[15 << 5];
    u_vec0[0] += sum;
  }
#endif

  for (i = 0; i < 32; i++) {
    gfloat sample = u_vec[i];
    if (sample > 0) {
      sample = sample * SCALE + 0.5f;
      if (sample < (SCALE - 1)) {
        samples[i] = (short) (sample);
      } else {
        samples[i] = (short) (SCALE - 1);
      }
    } else {
      sample = sample * SCALE - 0.5f;
      if (sample > -SCALE) {
        samples[i] = (short) (sample);
      } else {
        samples[i] = (short) (-SCALE);
      }
    }
  }
}
#endif

/* Synthesis matrixing variant which uses a 32 point DCT */
void
mp3_SubBandSynthesis (mp3tl * tl ATTR_UNUSED, frame_params * fr_ps,
    float *polyPhaseIn, gint channel, short *samples)
{
  gint i, j, k;
  gfloat *cur_synbuf = fr_ps->synbuf[channel];
  gfloat u_vec[HAN_SIZE];
  gfloat *u_vec0;

  /* Shift down 32 samples in the fifo, which should always leave room */
  k = fr_ps->bufOffset[channel];
  k = (k - 32) & 511;
  fr_ps->bufOffset[channel] = k;

  /* DCT part */
  MPG_DCT_32 (polyPhaseIn, cur_synbuf + k);

  /* Build the U vector */
  for (j = 0; j < 8; j++) {
    u_vec0 = u_vec + (j << 6);
    for (i = 0; i < 16; i++) {
      /* Copy first 32 elements */
      u_vec0 [i] = cur_synbuf [k + i + 16];
      u_vec0 [i + 17] = -cur_synbuf [k + 31 - i];
    }
    
    /* k wraps at the synthesis buffer boundary  */
    k = (k + 32) & 511;

    for (i = 0; i < 16; i++) {
      /* Copy next 32 elements */
      u_vec0 [i + 32] = -cur_synbuf [k + 16 - i];
      u_vec0 [i + 48] = -cur_synbuf [k + i];
    }
    u_vec0 [16] = 0;

    /* k wraps at the synthesis buffer boundary  */
    k = (k + 32) & 511;
  }
  
  /* Dewindow and output samples */
  mp3_dewindow_output (u_vec, samples, (gfloat*) dewindow);
}

#else
/* Original sub band synthesis function, for reference */
void
mp3_SubBandSynthesis (mp3tl * tl ATTR_UNUSED, frame_params * fr_ps,
    float *bandPtr, gint channel, short *samples)
{
  int i, j, k;
  float *bufOffsetPtr, sum;
  gint *bufOffset;
  float *synbuf = fr_ps->synbuf[channel];

  bufOffset = fr_ps->bufOffset;

  bufOffset[channel] = (bufOffset[channel] - 64) & 0x3ff;
  /* g_print ("bufOffset[%d] = %d\n", channel, bufOffset[channel]); */

  bufOffsetPtr = synbuf + bufOffset[channel];

  for (i = 0; i < 64; i++) {
    float *f_row = fr_ps->filter[i];
    sum = 0;
    for (k = 0; k < 32; k++)
      sum += bandPtr[k] * f_row[k];
    bufOffsetPtr[i] = sum;
  }

  /*  S(i,j) = D(j+32i) * U(j+32i+((i+1)>>1)*64)  */
  /*  samples(i,j) = MWindow(j+32i) * bufPtr(j+32i+((i+1)>>1)*64)  */
  for (j = 0; j < 32; j++) {
    sum = 0;
    sum += dewindow[j] * synbuf[(j + bufOffset[channel]) & 0x3ff];
    for (i = 64; i < 512; i += 64) {
      k = j + i;
      sum += dewindow[k] * synbuf[(k + i + bufOffset[channel]) & 0x3ff];
      sum +=
          dewindow[k - 32] * synbuf[(k - 32 + i + bufOffset[channel]) & 0x3ff];
    }

    /* Casting truncates towards zero for both positive and negative numbers,
       the result is cross-over distortion,  1995-07-12 shn */
    if (sum > 0) {
      sum = sum * SCALE + 0.5;
      if (sum < (SCALE - 1)) {
        samples[j] = (short) (sum);
      } else {
        samples[j] = (short) (SCALE - 1);
      }
    } else {
      sum = sum * SCALE - 0.5;
      if (sum > -SCALE) {
        samples[j] = (short) (sum);
      } else {
        samples[j] = (short) (-SCALE);
      }
    }
  }
}

#endif
#endif
